///
/// @author vay
/// @email hxhvay@126.com
/// @date 2020-05-09
///

library vaycore;

import 'package:vaycore/common/options.dart';
import 'package:vaycore/common/screen_adapter.dart';
import 'package:vaycore/dartin.dart';
import 'package:vaycore/utils.dart';

/// 初始化方法
typedef InitMethod = void Function();

/// 组件主类
class VayCore {
  /// 单例
  static VayCore _instance = new VayCore._internal();

  /// 私有构造函数
  VayCore._internal();

  /// 工厂构造函数
  factory VayCore() => _instance;

  /// 配置信息
  VayCoreOptions _options;

  VayCoreOptions get options => _options;

  /// 本地持久存储
  SpUtils _pref;

  SpUtils get pref => _pref;

  /// 加载
  void load(dynamic rootWidget,
      {bool isDebug, VayCoreOptions options, InitMethod initMethod}) async {
    if (options == null) {
      options = VayCoreOptions(isDebug: isDebug);
    }
    this._options = options;
    InnerWidgetsFlutterBinding.ensureInitialized();
    await _init();
    initMethod();
    InnerWidgetsFlutterBinding.ensureInitialized()
      ..attachRootWidget(rootWidget)
      ..scheduleWarmUpFrame();
  }

  _init() async {
    _pref = await SpUtils.getInstance();
    // 初始化需要依赖注入的模块
    if (options.modules?.isNotEmpty ?? false) {
      startDartIn(options.modules);
    }
  }
}

VayCore core = new VayCore();
